<?php

declare(strict_types=1);

namespace A2G\A2gProducts\Domain\Repository;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Context\Context;
use TYPO3\CMS\Extbase\Persistence\Generic\QueryResult;
use A2G\A2gProducts\Domain\Model\Filters;
use A2G\A2gProducts\Domain\Traits\{
    InjectFiltersRepositoryTrait,
    InjectFilterOptionsRepositoryTrait
};

/**
 * This file is part of the "Products" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2021 Raphael Martin <raphy.martin@gmail.com>, none
 */

/**
 * The repository for Products
 */
class ProductsRepository extends \TYPO3\CMS\Extbase\Persistence\Repository {

    use InjectFiltersRepositoryTrait;
    use InjectFilterOptionsRepositoryTrait;

    /**
     * @param array $categoryUids
     */
    public function getFromCategories(array $categoryUids, int $limit = 0, int $offset = 0, array $excludeProduct = [], bool $random = false) {
        if (!empty($categoryUids)) {
            $languageAspect = GeneralUtility::makeInstance(Context::class)->getAspect('language');
            $sys_language_uid = $languageAspect->getId();
            $query = $this->createQuery();
          
            $sql = ' SELECT DISTINCT pT.* FROM tx_a2gproducts_domain_model_products AS pT '
                    . 'JOIN tx_a2gproducts_domain_model_productvariants AS pvT ON pvT.product = pT.uid '
                    . 'JOIN tx_a2gproducts_products_categories_mm AS mmT ON mmT.uid_local = pT.uid ';
            $sqlWhere = ' WHERE pT.hidden=0 AND pT.deleted=0 AND pvT.hidden=0 AND pvT.deleted=0 AND mmT.uid_foreign IN (' . implode(',', $categoryUids) . ') AND pT.sys_language_uid IN (-1, :dcSysLangUid) ';
             $arguments = [
                ':dcSysLangUid' => $sys_language_uid
            ];

            if (!empty($excludeProduct)) {
                $sqlWhere .= ' AND pT.uid NOT IN (' . implode(',', $excludeProduct) . ') ';
            }

            if ($random) {
                $sqlWhere .= '  ORDER BY RAND() ';
            } else {
                $sqlWhere .= ' ORDER BY pT.sorting ';
            }



            if ($limit > 0) {
                $sqlWhere .= ' LIMIT :dcLimit ';
                $arguments[':dcLimit'] = $limit;
            }
            if ($offset > 0) {
                $sqlWhere .= ' OFFSET :dcOffset ';
                $arguments[':dcOffset'] = $offset;
            }

            $query->statement($sql . $sqlWhere, $arguments);
            return $query->execute();
        } else {
            return $this->findAll();
        }
    }

    /**
     * 
     * @param array $categoryUids
     * @param array $mainFilter
     * @param string $search
     * @param int $limit
     * @param int $offset
     * @param QueryResult $filters
     * @param bool $random
     * @return QueryResult
     */
    public function filter(array $categoryUids, array $mainFilter = [], string $search = '', int $limit = 0, int $offset = 0, QueryResult $filters = null, bool $random = false): QueryResult {
        $languageAspect = GeneralUtility::makeInstance(Context::class)->getAspect('language');
        $sys_language_uid = $languageAspect->getId();

        $query = $this->createQuery();
        $sql = 'SELECT DISTINCT pT.* FROM tx_a2gproducts_domain_model_products AS pT '
                . 'JOIN tx_a2gproducts_domain_model_productvariants AS pvT ON pvT.product = pT.uid '
                . 'JOIN tx_a2gproducts_products_categories_mm AS mmT ON mmT.uid_local = pT.uid ';
        $sqlWhere = 'WHERE pT.hidden=0 AND pT.deleted=0 AND pvT.hidden=0 AND pvT.deleted=0 AND pT.sys_language_uid IN (:dcSysLangUid,-1) ';
        if(!empty($categoryUids)){
               $sqlWhere .= 'AND mmT.uid_foreign IN (' . implode(',', $categoryUids) . ') ';
        }
        $arguments = [
            ':dcSysLangUid' => $sys_language_uid
        ];

        if (trim($search) !== '') {
            $sqlWhere .= 'AND pT.title LIKE :dcSearch ';
            $arguments[':dcSearch'] = '%' . $search . '%';
        }

        $filtersSortet = [];
        foreach ($filters as $filter) {
            $filtersSortet[$filter->getUid()] = $filter;
        }
        $x = 1;
        $i = 0;

        $sql1 = ' ';
        $sqlWhere1 = ' ';

        foreach ($mainFilter AS $key => $filter) {
            if($filtersSortet[$key] === null){
                $mainFilter[$key] = null;
            }
        }
        $mainFilter = array_filter($mainFilter);
        if (!empty($mainFilter)) {
            $sqlWhere .= '  AND ( ';
            foreach ($mainFilter AS $key => $filter) {
                $i = 0;
                if ($x > 1) {
                    $sqlWhere .= ' AND ';
                }
                $sqlWhere .= ' ( ';
//                $sqlWhere1 .= ' ( ';
                if (!empty($filter)) {
                    foreach ($filter AS $kay => $opt) {
                        if ($i > 0) {
                            $sqlWhere .= ' OR ';
                        } else {
//                            $sqlWhere .= ' ( fv' . $x . 'T.hidden = 0 OR fvpv' . $x . 'T.hidden = 0 OR sav' . $x . 'T.hidden = 0 OR savpv' . $x . 'T.hidden = 0 ) AND ( ';
                        }
                        $filterOpt = $this->filteroptionRepository->findByUid((int) $kay);
                        switch ($filtersSortet[$key]->getFilterType()) {
                            case 1:
                                // 'range ( uses min and max value )'
                                $sqlWhere .= ' ( fv' . $x . 'T.value ' . Filters::$dbOperations[$filtersSortet[$key]->getDbOperationMin()] . ' ' . $filterOpt->getValueMin() . ' ';
                                if ($filterOpt->getValueMax() > 0) {
                                    $sqlWhere .= ' AND fv' . $x . 'T.value ' . Filters::$dbOperations[$filtersSortet[$key]->getDbOperationMax()] . ' ' . $filterOpt->getValueMax();
                                }
                                $sqlWhere .= ' ) ';
                                break;
                            case 2:
                                // 'compare ( uses just min value )'
                                if (Filters::$dbOperations[$filtersSortet[$key]->getDbOperationMin()] === 'IN') {
                                    $sqlWhere .= '( fv' . $x . 'T.value IN (' . $filterOpt->getValueMin() . ' ) ';
                                } else {
                                    $sqlWhere .= '( fv' . $x . 'T.value ' . Filters::$dbOperations[$filtersSortet[$key]->getDbOperationMin()] . ' ' . $filterOpt->getValueMin() . ' ) ';
                                }
                                break;
                            case 3:
                                //  uses no value
                                $sqlWhere .= '( '
                                    . '( fv' . $x . 'T.rel_filter_option = ' . (int) $kay . ' AND fv' . $x . 'T.hidden = 0 ) '
                                    . 'OR ( fvpv' . $x . 'T.rel_filter_option = ' . (int) $kay . ' AND fvpv' . $x . 'T.hidden = 0 ) '
//                                    . 'OR ( sav' . $x . 'T.rel_filter_option = ' . (int) $kay . ' AND sav' . $x . 'T.hidden = 0 ) '
                                    . 'OR ( savpv' . $x . 'T.rel_filter_option = ' . (int) $kay . ' AND savpv' . $x . 'T.hidden = 0 ) '
                                    . ') ';
                                break;
                        }
                        $i = 1;
                    }

//                    $sqlWhere .= ' ) ';
                }
                $sql .= 'LEFT JOIN tx_a2gproducts_domain_model_filtervalues AS fv' . $x . 'T ON fv' . $x . 'T.products = pT.uid '
                        . 'LEFT JOIN tx_a2gproducts_domain_model_filtervalues AS fvpv' . $x . 'T ON fvpv' . $x . 'T.productvariants = pvT.uid '
//                        . 'LEFT JOIN tx_a2gproducts_domain_model_selectableattribute AS sav' . $x . 'T ON sav' . $x . 'T.products = pT.uid '
                        . 'LEFT JOIN tx_a2gproducts_domain_model_selectableattribute AS savpv' . $x . 'T ON savpv' . $x . 'T.productvariants = pvT.uid ';
                $arguments[':dcKey' . $x] = (int) $key;
                $x = $x + 1;
                $sqlWhere .= ' ) ';
            }
            $sqlWhere .= ' ) ';
        }

        if ($random) {
            $sqlWhere .= ' ORDER BY RAND()';
        } else {
            $sqlWhere .= ' ORDER BY pT.sorting';
        }

        if ($limit > 0) {
            $sqlWhere .= ' LIMIT :dcLimit ';
            $arguments[':dcLimit'] = $limit;
        }
        if ($offset > 0) {
            $sqlWhere .= ' OFFSET :dcOffset ';
            $arguments[':dcOffset'] = $offset;
        }
        $query->statement($sql . $sqlWhere, $arguments);
        return $query->execute();
    }

}
